#ifndef GAUSS_H

#define GAUSS_H

// ** This class is a Gaussian Random Number Generator
// ** A Gaussian number is expressed as having a mean and standard
// ** deviation and generates random numbers which conform to the standard
// ** bell curve distribution.  Additionally this class applies an optional
// ** minimum and maximum clamping range for all numbers generated.
// ** The Guassian class inherits the deterministic random number class.
// ** Therefore all gaussian sequences can be deterministic given the same
// ** initial random number seed.
// **
// ** The ASCII format for a gaussian number is as follows:
// **
// ** mean:std<min:max>
// **
// ** Examples of valid gaussian numbers.
// **
// **  30          (Means 30 with a standard deviation of 0)
// **  30:5        (Means 30 with a standard deviation of +/- 5)
// **  30:5<10>    (Means 30, stdev +/-5 minimum value of 10
// **  30:5<10:40> (Means 30, stdev +/-5, min value 10 max value 40)

#include "rand.h"

// Implementation of gaussian numbers.
class Gauss : public Rand
{
public:
  Gauss(int seed = 0);
  Gauss(const char *str,int seed=0);
  ~Gauss(void);

  float Get(void); // generate *deterministic* gaussian number.
  float GetCurrent(void) const; // last generated value.
  float GetMean(void) const;  // return mean.
  float GetStandardDeviation(void) const; // return standard deviation.
  void GetString(char *str) const; // get string representation


  void Set(float m,float s); // set mean and standard deviation.
  void Set(float m,float s,float min,float max); // set mean, standard deviation, min and max values.
  const char * Set(const char *arg); // set from asciiz string, returns pointer past gauss number.

  void SetMean(float m); // set just the mean.
  void SetStandardDeviation(float s); // set just the standard deviation.

  float RandGauss( void ); // construct and return gaussian number.

  // convert string to gaussian number.  Return code
  // indicates number of arguments found.
  int strtogmd(const char* spec,
               char** end,
               float& mean,
               float& deviation,
               float& min,
               float& max ) const;

  float GetMin(void) const { return mMin; };
  float GetMax(void) const { return mMax; };

private:
  float mMean;               // gaussian number has mean and
  float mStandardDeviation;  // standard deviation, also
  float mMin;                // min/max clamping values
  float mMax;

  float mCurrent;           // last got value.
  float mGauss1;            // 1st gaussian
  float mGauss2;            // 2nd gaussian
  bool  mSecond;            // have a computed second available.
};

#endif
